#![allow(clippy::println_empty_string)]
use std::{
    fs::File,
    io::{self, Write},
    path::PathBuf,
    u8,
};

extern crate png;

const MAX_ROWS: usize = 13;
const MAX_CHARS: u16 = 0x100; // Non inclusive, first char is 0x00

fn to_bitmap(path: &str, max_chars: u16, max_rows: usize) -> Result<Vec<u8>, io::Error> {
    let decoder = png::Decoder::new(File::open(path)?);
    let mut reader = decoder.read_info()?;
    let mut buf = vec![0; reader.output_buffer_size()];
    let info = reader.next_frame(&mut buf)?;
    let bytes = &buf[..info.buffer_size()];
    let compressed: Vec<u8> = bytes.iter().step_by(4).cloned().collect(); // We're not using color, so we skip duplicate color components
    let mut finalbuf = Vec::new();
    for (i, _bit) in compressed.iter().step_by(8).enumerate() {
        let mut byte = 0u8;
        for j in (0..8).rev() {
            if compressed[i * 8 + j] == 255 {
                byte |= 0x80 >> j;
            }
        }
        finalbuf.push(byte);
    }
    Ok(finalbuf)
}

fn reorder_bitmap(bitmap: &[u8]) -> Vec<u8> {
    let mut reordered: Vec<u8> = Vec::new();
    for row in 0..MAX_ROWS {
        for character in 0x00..MAX_CHARS {
            let mut byte = 0u8;
            for i in 0..8 {
                // character number + row offset & bitmask = pixel
                if bitmap[character as usize + (row * 0xFF)] & 0x80 >> i > 0 {
                    byte |= 0x80 >> i;
                }
            }
            reordered.push(byte);
        }
    }
    reordered
}

fn main() -> std::io::Result<()> {
    let compressed = to_bitmap("./cozette.png", MAX_CHARS, MAX_ROWS)?;
    let reordered = reorder_bitmap(&compressed);
    let mut outfile = File::create("./cozette.bin")?;
    outfile.set_len(0xFFFF)?;
    outfile.write_all(&reordered)?;
    Ok(())
}
